/* -*- Mode: js; indent-tabs-mode: nil; js-basic-offset: 4; tab-width: 4; -*- */
/*
 * This file is part of Looking Glass Button.
 * https://gitlab.gnome.org/glerro/gnome-shell-extension-lgbutton
 *
 * prefs.js
 *
 * Copyright (c) 2019-2023 Gianni Lerro {glerro} ~ <glerro@pm.me>
 *
 * Looking Glass Button is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Looking Glass Button is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Looking Glass Button. If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 * SPDX-FileCopyrightText: 2019-2023 Gianni Lerro <glerro@pm.me>
 */

/* exported init buildPrefsWidget fillPreferencesWindow */

'use strict';

const {GObject, Gio, Gdk, Gtk} = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const Config = imports.misc.config;
const shellVersion = parseFloat(Config.PACKAGE_VERSION);

const Adw = shellVersion >= 42 ? imports.gi.Adw : null;

const TEMPLATE_GTK = shellVersion < 40 ? 'prefs_gtk3.ui' : 'prefs_gtk4.ui';
const TEMPLATE_ADW = 'prefs_adw1.ui';

// Prefs Widget for Gnome Shell < 42, work with gtk3 and gtk4
const LGButtonPrefsWidgetGtk = GObject.registerClass({
    GTypeName: 'LGButtonPrefsWidgetGtk',
    Template: Me.dir.get_child(TEMPLATE_GTK).get_uri(),
    InternalChildren: ['localExtensionsFolder', 'systemExtensionsFolder',
        'localThemesFolder', 'systemThemesFolder'],
}, class LGButtonPrefsWidgetGtk extends Gtk.Box {
    _init(params = {}) {
        super._init(params);

        let cssProvider = new Gtk.CssProvider();

        cssProvider.load_from_path(`${Me.dir.get_path()}/prefs.css`);
        Gtk.StyleContext.add_provider_for_display(Gdk.Display.get_default(),
            cssProvider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);

        this._settings = ExtensionUtils.getSettings();

        this._settings.bind('local-extensions-folder', this._localExtensionsFolder,
            'active', Gio.SettingsBindFlags.DEFAULT);

        this._settings.bind('system-extensions-folder', this._systemExtensionsFolder,
            'active', Gio.SettingsBindFlags.DEFAULT);

        this._settings.bind('local-themes-folder', this._localThemesFolder,
            'active', Gio.SettingsBindFlags.DEFAULT);

        this._settings.bind('system-themes-folder', this._systemThemesFolder,
            'active', Gio.SettingsBindFlags.DEFAULT);

        this.connect('destroy', () => this._settings.run_dispose());
    }

    _onExtensionsListBoxActivated(listBox, listBoxRow) {
        if (listBoxRow.get_index() === 0)
            this._switchToggle(this._localExtensionsFolder);
        else if (listBoxRow.get_index() === 2)
            this._switchToggle(this._systemExtensionsFolder);
    }

    _onThemesListBoxActivated(listBox, listBoxRow) {
        if (listBoxRow.get_index() === 0)
            this._switchToggle(this._localThemesFolder);
        else if (listBoxRow.get_index() === 2)
            this._switchToggle(this._systemThemesFolder);
    }

    _switchToggle(gtkSwitch) {
        this._gtkSwitch = gtkSwitch;
        if (this._gtkSwitch.get_active())
            this._gtkSwitch.set_active(false);
        else
            this._gtkSwitch.set_active(true);
    }
});

// Prefs Widget for Gnome Shell >= 42, work with adw1
let LGButtonPrefsWidgetAdw = null;
try {
    LGButtonPrefsWidgetAdw = GObject.registerClass({
        GTypeName: 'LGButtonPrefsWidgetAdw',
        Template: Me.dir.get_child(TEMPLATE_ADW).get_uri(),
        InternalChildren: ['localExtensionsFolder', 'systemExtensionsFolder',
            'localThemesFolder', 'systemThemesFolder'],
    }, class LGButtonPrefsWidgetAdw extends Adw.PreferencesPage {
        _init(params = {}) {
            super._init(params);

            this._settings = ExtensionUtils.getSettings();

            this._settings.bind('local-extensions-folder', this._localExtensionsFolder,
                'active', Gio.SettingsBindFlags.DEFAULT);

            this._settings.bind('system-extensions-folder', this._systemExtensionsFolder,
                'active', Gio.SettingsBindFlags.DEFAULT);

            this._settings.bind('local-themes-folder', this._localThemesFolder,
                'active', Gio.SettingsBindFlags.DEFAULT);

            this._settings.bind('system-themes-folder', this._systemThemesFolder,
                'active', Gio.SettingsBindFlags.DEFAULT);

            this.connect('destroy', () => this._settings.run_dispose());
        }
    });
} catch (error) {}

/**
 * This function is called once when the Preferencs Windows is loaded.
 *
 * @function init
 */
function init() {
    // Inizializing translations
    ExtensionUtils.initTranslations();
}

/**
 * This function build the Preferencs Windows for Gnome Shell < 42
 *
 * @function buildPrefsWidget
 * @returns {Gtk.Widget} - the prefs widget
 */
function buildPrefsWidget() {
    let prefsWidgetGtk = new LGButtonPrefsWidgetGtk();

    if (prefsWidgetGtk.show_all) {
        prefsWidgetGtk.show_all();
    } else {
        prefsWidgetGtk.connect('realize', () => {
            let window = prefsWidgetGtk.get_root();
            window.default_width = 640;
            window.default_height = 450;
        });
    }

    return prefsWidgetGtk;
}

/**
 * This function fill prefs window {Adw.PreferencesWindow} and have higher priority to builPrefsWidget().
 * Work with Gnome Shell >= 42
 *
 * @function fillPreferencesWindow
 * @param {Adw.PreferencesWindow} window - the prefs window.
 * @returns {void}
 */
function fillPreferencesWindow(window) {
    if (Adw) {
        let prefsWidgetAdw = new LGButtonPrefsWidgetAdw();
        window.add(prefsWidgetAdw);
        window.set_default_size(640, 450);
    }
}

